<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Service.php,v 1.2 2005/04/15 13:07:46 mike Exp $
 */

/**
 * Requires Gadget_Plugin
 */
class_exists('Gadget_Plugin') or require_once 'Gadget/Plugin.php';

/**
 * Gadget_Service configurations
 * @ignore
 */
$GLOBALS['_Gadget'][GADGET_SERVICE]['Config'] = array();

/**
 * Gadget_Service registry
 * @ignore
 */
$GLOBALS['_Gadget'][GADGET_SERVICE]['Registry'] = array(
    SERVICE_LOG         => array(
        'default'   => array('Gadget_Service_Log', null),
        'userdef'   => array(),
    ),
    SERVICE_RIGHTS  => array(
        'default'   => array('Gadget_Service_Rights', null),
        'userdef'   => array(),
    ),
    SERVICE_AUTH        => array(
        'default'   => array('Gadget_Service_Auth', null),
        'userdef'   => array(),
    ),
    SERVICE_AUTOLOGIN   => array(
        'default'   => array('Gadget_Service_Autologin', null),
        'userdef'   => array(),
    ),
    SERVICE_ERRORDISPLAY=> array(
        'default'   => array('Gadget_Service_Errordisplay', null),
        'userdef'   => array(),
    ),
    SERVICE_ERRORNOTIFY => array(
        'default'   => array('Gadget_Service_Errornotify', null),
        'userdef'   => array(),
    ),
    SERVICE_ERRORLOG    => array(
        'default'   => array('Gadget_Service_Errorlog', null),
        'userdef'   => array(),
    ),
    SERVICE_SECURITY    => array(
        'default'   => array('Gadget_Service_Security', null),
        'userdef'   => array(),
    ),
    SERVICE_MAILACCOUNT => array(
        'default'   => array('Gadget_Service_Mailaccount', null),
        'userdef'   => array(),
    ),
    SERVICE_RENDER      => array(
        'default'   => array('Gadget_Service_Render', null),
        'userdef'   => array(),
    ),
    SERVICE_STATS       => array(
        'default'   => array('Gadget_Service_Stats', null),
        'userdef'   => array(),
    ),
);

/**
 * Gadget_Service
 *
 * Gadget's service base class.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service extends Gadget_Plugin
{
    /**
     * Plugin Type
     * 
     * @var string
     */
    var $plugin = GADGET_SERVICE;
    
    /**
     * Service
     * 
     * @var string
     */
    var $service;
    
    /**
     * Singleton
     * 
     * @static
     * @access  public
     * @return  object
     * @param   string
     */
    function &singleton($service)
    {
        return parent::singleton(GADGET_SERVICE, $service);
    }
    
    /**
     * Configure a Service
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $service
     * @param   mixed   $params
     */
    function configure($service, $params)
    {
        return parent::configure(GADGET_SERVICE, $service, $params);
    }
    
    /**
     * Register Service
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $service
     * @param   string  $class
     * @param   string  $file
     */
    function register($service, $class, $file = null)
    {
        return parent::register(GADGET_SERVICE, $service, $class, $file);
    }
    
    /**
     * Lookup
     * 
     * @static
     * @access  public
     * @return  array
     * @param   string  $service
     */
    function lookup($service)
    {
        return parent::lookup(GADGET_SERVICE, $service);
    }
    
    /**
     * Run the Service
     * 
     * @access  public
     * @return  bool
     * @param   mixed   $args, ...
     */
    function run()
    {
        $arguments = func_get_args();
        if (call_user_func_array(array(&$this, 'execute'), $arguments)) {
            return $this->onSuccess();
        } else {
            return $this->onFailure();
        }
    }
    
    /**
     * On Success
     * 
     * @access  protected
     * @return  bool true
     */
    function onSuccess($log = null)
    {
        $message = $log ? $log : Gadget::currentUser('alias');
        Gadget::log($this->service, '[SUCCESS] '. $message);
        return true;
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  bool false
     */
    function onFailure($log = null)
    {
        $message = $log ? $log : Gadget::currentUser('alias');
        Gadget::log($this->service, '[FAILURE] '. $message);
        return false;
    }
}
?>
