<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.4 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Search.php,v 1.4 2005/04/21 15:24:13 mike Exp $
 */

/**
 * Requires Gadget_Tree_Search_Condition
 */
require_once 'Gadget/Tree/Search/Condition.php';

/** 
 * Gadget_Tree_Search
 *
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Search
{
    /**
     * Path
     * 
     * @var string
     */
    var $path = '';
    
    /**
     * Separator
     * 
     * @var string
     */
    var $sep  = '';
    
    /**
     * Conditions
     * 
     * @var array
     */
    var $conditions = array();
    
    /**
     * Constructor
     * 
     * @param   string  $path
     * @param   string  $sep
     */
    function Gadget_Tree_Search($path, $sep = '/')
    {
        $this->sep = $sep;
        $this->path = $path;
    }
    
    /**
     * Add Condition
     * 
     * @access  public
     * @return  bool
     * @param   object  Gadget_Tree_Search_Condition $condition
     */
    function addCondition(&$condition)
    {
        if ($isCond = is_a($condition, 'Gadget_Tree_Search_Condition')) {
            $this->conditions[] = &$condition;
        }
        return $isCond;
    }
    
    /**
     * Parse
     * 
     * @access  public
     * @return  void
     */
    function parse()
    {
        foreach (explode($this->sep, $this->path) as $part) {
            if ($part) {
                $this->addCondition(new Gadget_Tree_Search_Condition($part));
            }
        }
    }
    
    /**
     * Search
     * 
     * @access  public
     * @return  mixed
     * @param   object  Gadget_Tree_Node $node
     */
    function &search(&$node)
    {
        return Gadget_Tree_Search::findNode($node, $this->conditions);
    }
    
    /**
     * Find Child
     * 
     * @static
     * @access  public
     * @return  object  Gadget_Tree_Node
     * @param   object  Gadget_Tree_Node $node
     * @param   object  Gadget_Tree_Search_Condition $condition
     */
    function &findChild(&$node, &$condition)
    {
        $found = null;
        for ($i = 0, $c = $node->count(); $i < $c && !$found; $i++) {
            if ($condition->testNode($node->childAt($i))) {
                $found = &$node->childAt($i);
            }
        }
        return $found;
    }
    
    /**
     * Find Node
     * 
     * @static
     * @access  public
     * @return  object  Gadget_Tree_Node
     * @param   object  Gadget_Tree_Node $node
     * @param   array   $conditions
     */
    function &findNode(&$node, &$conditions)
    {
        $found = null;
        
        if (!$conditions) {
            return $node;
        }
        
        if ($conditions[0]->testNode($node)) {
            $found = &$node;
            
            for ($i = 1, $c = count($conditions); $i < $c && $found; $i++) {
                $found = &Gadget_Tree_Search::findChild($found, $conditions[$i]);
            }
        }
        
        return $found;
    }
    
    /**
     * Compare
     * 
     * @static
     * @access  public
     * @return  bool
     * @param   string  $op
     * @param   mixed   $left
     * @param   mixed   $right
     */
    function compare($op, $left, $right)
    {
        /*#*/echo "[$left] [$op] [$right]\n";
        switch ($op)
        {
            case '=':   return $left == $right;
            case '>':   return $left > $right;
            case '<':   return $left < $right;
            case '>=':  return $left >= $right;
            case '<=':  return $left <= $right;
            case '!=':  return $left != $right;
            default:
                Gadget_Tree::raiseError("Invalid comparison operator $op");
                return false;
        }
    }
    
}

?>
