<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Condition.php,v 1.2 2005/04/21 15:24:13 mike Exp $
 */

/** 
 * Gadget_Tree_Search_Condition
 *
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Search_Condition
{
    /**
     * Axis Tests
     * 
     * @var object
     */
    var $axisTest;
    
    /**
     * Node Tests
     * 
     * @var array
     */
    var $nodeTests = array();
    
    /**
     * Gadget_Tree_Search_Condition
     * 
     * @access  public
     * @return  void
     * @param   string  $condition
     */
    function Gadget_Tree_Search_Condition($condition = null)
    {
        if ($condition) {
            $this->parseNodeAxis($condition);
            $this->parseNodeTest($condition);
        }
    }
    
    /**
     * Parse Node Axis
     * 
     * @access  public
     * @return  void
     * @param   string  $axis
     */
    function parseNodeAxis($axis)
    {
        class_exists('Gadget_Tree_Search_AxisTest') or 
            require_once 'Gadget/Tree/Search/AxisTest.php';
        $axis = substr($axis, 0, strcspn($axis, '[('));
        $this->axisTest = &new Gadget_Tree_Search_AxisTest($axis);
    }
    
    /**
     * Parse Node Test
     * 
     * @access  public
     * @return  void
     * @param   string  $test
     */
    function parseNodeTest($test)
    {
        class_exists('Gadget_Tree_Search_NodeTest') or 
            require_once 'Gadget/Tree/Search/NodeTest.php';
        
        
        if (preg_match_all('/\[([^\[\]]+)\]/', $test, $matches)) {
            foreach ($matches[1] as $c) {
                $attribute = $c{0} == '@';
                $this->nodeTests[] = &new Gadget_Tree_Search_NodeTest(
                    $attribute ? 'attribute' : 'function', array_map('trim', 
                    preg_split('/([!=><]+)/', $c, 2, PREG_SPLIT_DELIM_CAPTURE))
                );
            }
        }
    }
    
    /**
     * Test Node
     * 
     * @access  public
     * @return  bool
     * @param   object  Gadget_Tree_Node $node
     */
    function testNode(&$node)
    {
        if ($this->axisTest && !$this->axisTest->testNode($node)) {
            return false;
        }
        for ($i = 0, $c = count($this->nodeTests); $i < $c; $i++) {
            if (!$this->nodeTests[$i]->testNode($node)) {
                return false;
            }
        }
        return true;
    }
}

?>
